import TurndownService from "turndown";

// https://gitdab.com/cadence/out-of-your-element/src/commit/c9aa5b4c64ac9f47dc75059950aee67e8244b575/m2d/converters/event-to-message.js

export const Turndown = new TurndownService({
  hr: "----",
  headingStyle: "atx",
  codeBlockStyle: "fenced",
});

Turndown.remove("mx-reply" as any);

Turndown.addRule("strikethrough", {
  filter: ["del", "s"],
  replacement(content) {
    return `~~${content}~~`;
  },
});

Turndown.addRule("underline", {
  filter: ["u"],
  replacement(content) {
    return `__${content}__`;
  },
});

Turndown.addRule("blockquote", {
  filter: "blockquote",
  replacement(content) {
    content = content.replace(/^\n+|\n+$/g, "");
    content = content.replace(/^/gm, "> ");
    return content;
  },
});

Turndown.addRule("spoiler", {
  filter: function (node, options) {
    return node.tagName === "SPAN" && node.hasAttribute("data-mx-spoiler");
  },

  replacement(content, node_) {
    let node: HTMLElement = node_ as any;

    if (node.getAttribute("data-mx-spoiler")) {
      // escape parentheses so it can't become a link
      return `\\(${node.getAttribute("data-mx-spoiler")}\\) ||${content}||`;
    }

    return `||${content}||`;
  },
});

Turndown.addRule("inlineLink", {
  filter(node, options) {
    return node.nodeName === "A" && !!node.getAttribute("href");
  },

  replacement(content, node_) {
    let node: HTMLElement = node_ as any;

    const href = node.getAttribute("href");
    if (href) {
      if (
        href.startsWith("https://matrix.to/#/@_discord_") &&
        href.indexOf("@_discord_bot") === -1
      ) {
        return `<@${href.replace("https://matrix.to/#/@_discord_", "").replace(":" + process.env.MATRIX_HOMESERVER, "")}>`;
      }

      content = content.replace(/ @.*/, "");
      if (href === content) return href;
      if (href.startsWith("https://matrix.to/#/@") && content[0] !== "@")
        content = "@" + content;
    }

    return "[" + content + "](" + href + ")";
  },
});

Turndown.addRule("listItem", {
  filter: "li",
  replacement: function (content, node_, options) {
    let node: HTMLElement = node_ as any;

    content = content
      .replace(/^\n+/, "") // remove leading newlines
      .replace(/\n+$/, "\n") // replace trailing newlines with just a single one
      .replace(/\n/gm, "\n  "); // indent
    var prefix = options.bulletListMarker + " ";
    var parent = node.parentNode;
    if (parent?.nodeName === "OL") {
      var start = (parent as HTMLElement).getAttribute("start");
      var index = Array.prototype.indexOf.call(parent.children, node);
      prefix = (start ? Number(start) + index : index + 1) + ". ";
    }
    return (
      prefix + content + (node.nextSibling && !/\n$/.test(content) ? "\n" : "")
    );
  },
});
